from ftplib import FTP
import sys
from os import stat
from string import Template
from datetime import datetime
import time
import git
import pytz
from zipfile import ZipFile
from texttable import Texttable
from myconfig import ftp_server, ftp_user, ftp_pass, remotepath, DEBUG, repodir

# this script grabs Metro's latest GTFS archive for Nextrail,
# unpacks the archive and builds a README.md file.
# it then unpacks the files, commits them into the repository
# and pushes the changes to https://gitlab.com/LACMTA/gtfs_rail

myoutf="gtfs_rail.zip"
r = git.Repo(repodir)

ftp = FTP(ftp_server)
ftp.login(ftp_user, ftp_pass)

def writeREADME(tpl='README.tpl', outfile='README.md', thevars={ 'thefiletable':'', 'thetimestamp':'', 'thegitlog':'' }):
    #open the template file
    template_file = open( tpl )
    src = Template( template_file.read() )
    #do the substitution
    README = src.substitute(thevars)

    f = open(outfile,"w")
    f.write( README )
    f.close()

def dos2unix(afile,DEBUG=False):
    content = ''
    outsize = 0
    with open(afile, 'rb') as infile:
        content = infile.read()
    infile.close()
    with open(afile, 'wb') as outfile:
        for line in content.splitlines():
            outsize += len(line) + 1
            outfile.write(line + b'\n')
    outfile.close()
    if DEBUG:
        print("Done. Stripped %s bytes." % (len(content)-outsize))

def getbinary(ftp, remotedir="/", remotef="hi.zip", outfile=None):
    # fetch a binary file
    ftp.cwd(remotedir)
    if outfile is None:
        outfile = sys.stdout
    else:
        with open(outfile, 'wb') as f:
            ftp.retrbinary("RETR " + remotef, f.write)

resp = getbinary(ftp, remotedir=remotepath, remotef="GTFS_Rail_Nextrain.zip", outfile="gtfs_rail.zip")
if DEBUG: print( "ftp server reponse: %s" %(resp))
ftp.quit()

# let's work with the zip file
zobj = ZipFile(myoutf)
stats = stat(myoutf)
dt = datetime.fromtimestamp(stats.st_ctime)

tzstr = "America/Los_Angeles"
fmt = '%Y-%m-%d %H:%M:%S %Z'
dt = datetime.now(pytz.timezone(tzstr))
version = dt.strftime(fmt) + ' ' + tzstr

# unzip the contents
zobj.extractall()

# let's update the README.md file
flist = zobj.namelist()

table = Texttable()
# horizontal lines, vertical lines, intersection points of these lines, and the header line
table.set_chars(['','','','-'])
table.set_deco(table.HEADER | table.VLINES)
table.set_cols_align(["r", "c", "l"])
tablelist = []
tablelist.append(["Length", "Creation datetime", "Name"])

# get the contents and add the files to the commit list
for fi in flist:
    dos2unix(fi,DEBUG)
    r.index.add([fi])
    dtup = zobj.getinfo(fi).date_time
    dtstr = "%d-%02d-%02d %02d:%02d" %(dtup[0],dtup[1],dtup[2],dtup[3],dtup[4])
    thisrow = [zobj.getinfo(fi).file_size, dtstr, fi]
    tablelist.append(thisrow)

table.add_rows(tablelist)
thefiletable = table.draw()
if DEBUG: print(thefiletable)

# git log -1  --stat
thegitlog = r.git.log('-1', '--stat')

tplvars = {
    'thefiletable':thefiletable,
    'thetimestamp':version,
    'thegitlog':thegitlog,
    }

writeREADME(tpl='README.tpl', outfile='README.md', thevars=tplvars)

# did more than one file (the README.me) change?
files_changed=r.git.diff('--name-only')
if (".zip" in files_changed):
    # add the README.md file to the commit list
    r.index.add(["README.md"])

    # add the original ZIPFILE file to the commit list
    r.index.add([myoutf])

    # commit all the changes
    r.index.commit(version)

    #pull and push
    origin = r.remotes.origin
    pullinfo = origin.pull()
    # if DEBUG: print (pullinfo)
    pushinfo = origin.push()
    # if DEBUG: print (pushinfo)
